clear all

% link with bl-ICON's source
addpath('bl_icon_SRC')


%% use synthetic data
%[In_D,In_A,dt,dD,c_D,c_A,q_D,q_A,units_I,units_t] = generate_synthetic_data();

%% or import experimental data (and set options manually)
In_D = load('example_trace_donor.txt');
In_A = load('example_trace_acceptor.txt');

units_t = 's';
units_I = 'photon';
dt = 0.1;
dD = 0.099;
c_D = 0.90;
c_A = 0.75;
q_D = 0.85;
q_A = 0.75;

%% initialize MCMC chain
opts.In_D = In_D;   % measured intensities in the donor channel
opts.In_A = In_A;   % measured intensities in the acceptor channel
opts.dt = dt;       % frame period
opts.dD = dD;       % exposure period
opts.c_D = c_D;     % cross-talk coefficient D -> D
opts.c_A = c_A;     % cross-talk coefficient A -> A
opts.q_D = q_D;     % quantum efficiency for D
opts.q_A = q_A;     % quantum efficiency for A
opts.units_t = units_t; % units of time
opts.units_I = units_I; % units of intensity


chain = chainer_main([],0,opts,[],[]);
clear opts


%% expand MCMC chain
chain = chainer_main(chain,750,[],true,true);

if chain.sizeGB > 0.5
    chain = chainer_main(chain,-fix(chain.length/2),[],true,[]);
end

%% show results
show_estimates_efficiency      (chain,0.30,1,200)
show_estimates_emission_rates  (chain,0.30,1,300)
show_estimates_trace_efficiency(chain,0.30,1,400)


