function params = chainer_init_params(opts)


%% Sizes
if isfield(opts, 'M')
    params.M = opts.M;
else
    params.M = 25;              % weak limit
end

params.N = length(opts.In_D);   % signal length


%% Data and units
params.In_D = reshape(opts.In_D,1,params.N); % measured intensities in the donor channel
params.In_A = reshape(opts.In_A,1,params.N); % measured intensities in the acceptor channel

params.units_I = opts.units_I; % units of intensity, i.e. [I]
params.units_t = opts.units_t; % units of time, i.e. [t]

params.dt = opts.dt;  % acquisition period, [t]
params.dD = opts.dD;  % exposure period, [t]

%% Cross-talk coefficients
params.cDD = opts.c_D;      % D -> D
params.cDA = 1-params.cDD;  % D -> A

params.cAA = opts.c_A;      % A -> A
params.cAD = 1-params.cAA;  % A -> D

%% Quantum efficiency
params.qD = opts.q_D;      % D
params.qA = opts.q_A;      % A

%% Priors

% transition probs and base

if isfield(opts, 'alpha')
    params.alpha = opts.alpha;
else
    params.alpha = 1;
end  
    
if isfield(opts, 'gamma')
    params.gamma = opts.gamma;
else
    params.gamma = 1;
end 

% photophysics [0,1,*]

if isfield(opts, 'wi_D_prior_eta')
    params.wi_D_prior_eta = opts.wi_D_prior_eta;
else
    params.wi_D_prior_eta  = [1 1 1];
end 

if isfield(opts, 'wi_D_prior_zeta')
    params.wi_D_prior_zeta = opts.wi_D_prior_zeta;
else
    params.wi_D_prior_zeta = [1 1 1];
end

if isfield(opts, 'wi_A_prior_eta')
    params.wi_A_prior_eta = opts.wi_A_prior_eta;
else
    params.wi_A_prior_eta  = [1 1 1];
end

if isfield(opts, 'wi_A_prior_zeta')
    params.wi_A_prior_zeta = opts.wi_A_prior_zeta;
else
    params.wi_A_prior_zeta = [1 1 1];
end


% baseline emission rate

if isfield(opts, 'tht_prior_phi')
    params.tht_prior_phi = opts.tht_prior_phi;
else
    params.tht_prior_phi = 2;
end 

if isfield(opts, 'tht_prior_psi')
    params.tht_prior_psi = opts.tht_prior_psi;
else
    params.tht_prior_psi = 0.5*sum(params.In_D+params.In_A)/(params.N*params.dD);   % [I]/[t]
end

% background emission multipliers

if isfield(opts, 'rho_D_prior_phi')
    params.rho_D_prior_phi = opts.rho_D_prior_phi;
else
    params.rho_D_prior_phi = 1;
end

if isfield(opts, 'rho_D_prior_psi')
    params.rho_D_prior_psi = opts.rho_D_prior_psi;
else
    params.rho_D_prior_psi = 1;
end

if isfield(opts, 'rho_A_prior_phi')
    params.rho_A_prior_phi = opts.rho_A_prior_phi;
else
    params.rho_A_prior_phi = 1;
end 

if isfield(opts, 'rho_A_prior_psi')
    params.rho_A_prior_psi = opts.rho_A_prior_psi;
else
    params.rho_A_prior_psi = 1;
end

% molecule emission multipliers
if isfield(opts, 'kap_D_prior_phi')
    params.kap_D_prior_phi = opts.kap_D_prior_phi;
else
    params.kap_D_prior_phi = 1;
end
    
if isfield(opts, 'kap_D_prior_psi')
    params.kap_D_prior_psi = opts.kap_D_prior_psi;
else
    params.kap_D_prior_psi = 1;
end

if isfield(opts, 'kap_A_prior_phi')
    params.kap_A_prior_phi = opts.kap_A_prior_phi;
else
    params.kap_A_prior_phi = 1;
end 

if isfield(opts, 'kap_A_prior_psi')
    params.kap_A_prior_psi = opts.kap_A_prior_psi;
else
    params.kap_A_prior_psi = 1;
end

if isfield(opts, 'kap_S_prior_phi')
    params.kap_S_prior_phi = opts.kap_S_prior_phi;
else
    params.kap_S_prior_phi = 1;
end
    
if isfield(opts, 'kap_S_prior_psi')
    params.kap_S_prior_psi = opts.kap_S_prior_psi;
else
    params.kap_S_prior_psi = 1;
end



%% MCMC
if isfield(opts, 'HMC_eps')
    params.HMC_eps = opts.HMC_eps;
else
    params.HMC_eps = 0.0001;
end

if isfield(opts, 'HMC_L')
    params.HMC_L = opts.HMC_L;
else
    params.HMC_L = 125;
end


if isfield(opts, 'MH_a_D')
    params.MH_a_D = opts.MH_a_D;
else
    params.MH_a_D = 250;
end

if isfield(opts, 'MH_a_A')
    params.MH_a_A = opts.MH_a_A;
else
    params.MH_a_A = 250;
end


%% Other params
params.i_skip = 1;

params.rep_bm  = 15;
params.rep_tht =  5;
params.rep_rho =  5;


