function chain = chainer_main(chain_init,d_length,opts,flag_status,flag_visual)
% to init:
% chain = chainer_main([]   ,  0, opts, true, []  );
% to expand:
% chain = chainer_main(chain,+25, []  , true, true);
% to reduce:
% chain = chainer_main(chain,-10, []  , true, []  );

rng('shuffle')


% init chain --------------------------------------------------------------
if d_length == 0
    tic_id = tic;

    % MCMC
    chain.params = chainer_init_params(opts);
    chain.length = 1;
    chain.sizeGB = nan;
    chain.sample = [];

    chain.sample = sampler_init_sample(chain.params,opts);
    
    % history
    chain.i     = cast(chain.sample.i     , 'uint64' );
    chain.sn    = cast(chain.sample.sn    , get_int_type(chain.params.N)  );
    chain.fn_D  = cast(chain.sample.fn_D  , 'logical' );
    chain.fn_A  = cast(chain.sample.fn_A  , 'logical' );
    chain.wi_D  = cast(chain.sample.wi_D  , 'single'  );
    chain.wi_A  = cast(chain.sample.wi_A  , 'single'  );
    chain.pm    = cast(chain.sample.pm(:)', 'single'  );
    chain.ps    = cast(chain.sample.ps    , 'single'  );
    chain.ksi_D = cast(chain.sample.ksi_D , 'single'  );
    chain.ksi_A = cast(chain.sample.ksi_A , 'single'  );
    chain.lam_S = cast(chain.sample.lam_S , 'single'  );
    chain.lam_D = cast(chain.sample.lam_D , 'single'  );
    chain.lam_A = cast(chain.sample.lam_A , 'single'  );
    chain.eff   = cast(chain.sample.eff   , 'single'  );

    
    if flag_status
        disp(['CHAINER: chain initiated (total time = ',num2str(toc(tic_id)),' s)'])
    end


% expand chain ------------------------------------------------------------
elseif d_length > 0
    tic_id = tic;

    chain.params = chain_init.params;
    chain.length = chain_init.length + d_length;
    chain.sizeGB = nan;
    chain.sample = chain_init.sample;
    
    chain.i     = [ chain_init.i     ; zeros( d_length, 1               , 'like', chain_init.i    )];
    chain.sn    = [ chain_init.sn    ; zeros( d_length, chain.params.N  , 'like', chain_init.sn   )];
    chain.fn_D  = [ chain_init.fn_D  ; false( d_length, chain.params.N                            )];
    chain.fn_A  = [ chain_init.fn_A  ; false( d_length, chain.params.N                            )];
    chain.wi_D  = [ chain_init.wi_D  ;   nan( d_length, 3               , 'like', chain_init.wi_D )];
    chain.wi_A  = [ chain_init.wi_A  ;   nan( d_length, 3               , 'like', chain_init.wi_A )];
    chain.pm    = [ chain_init.pm    ;   nan( d_length, chain.params.M^2, 'like', chain_init.pm   )];
    chain.ps    = [ chain_init.ps    ;   nan( d_length, chain.params.M  , 'like', chain_init.ps   )];
    chain.ksi_D = [ chain_init.ksi_D ;   nan( d_length, 1               , 'like', chain_init.ksi_D)];
    chain.ksi_A = [ chain_init.ksi_A ;   nan( d_length, 1               , 'like', chain_init.ksi_A)];
    chain.lam_S = [ chain_init.lam_S ;   nan( d_length, 1               , 'like', chain_init.lam_S)];
    chain.lam_D = [ chain_init.lam_D ;   nan( d_length, chain.params.M  , 'like', chain_init.lam_D)];
    chain.lam_A = [ chain_init.lam_A ;   nan( d_length, chain.params.M  , 'like', chain_init.lam_A)];
    chain.eff   = [ chain_init.eff   ;   nan( d_length, chain.params.M  , 'like', chain_init.eff  )];
    
    
    if flag_visual
        Gim = chainer_visualize([],chain);
    end
    
    %---------------------------- expand chain
    r = chain_init.length+1;
    while r <= chain.length
        
        chain.sample = sampler_update(chain.sample,chain.params);
        
        if mod(chain.sample.i,chain.params.i_skip) == 0
            
            chain.i    (r  ) = chain.sample.i;
            chain.sn   (r,:) = chain.sample.sn;
            chain.fn_D (r,:) = chain.sample.fn_D;
            chain.fn_A (r,:) = chain.sample.fn_A;
            chain.wi_D (r,:) = chain.sample.wi_D;
            chain.wi_A (r,:) = chain.sample.wi_A;
            chain.pm   (r,:) = chain.sample.pm(:)';
            chain.ps   (r,:) = chain.sample.ps;
            chain.ksi_D(r  ) = chain.sample.ksi_D;
            chain.ksi_A(r  ) = chain.sample.ksi_A;
            chain.lam_S(r  ) = chain.sample.lam_S;
            chain.lam_D(r,:) = chain.sample.lam_D;
            chain.lam_A(r,:) = chain.sample.lam_A;
            chain.eff  (r,:) = chain.sample.eff;
            
            if flag_visual
                chainer_visualize(Gim,chain);
            end
            
            if flag_status
                disp([  'i = ', num2str(chain.sample.i,'%d'), ...
                     ' - HMC acc = ', num2str( chain.sample.rec(1,1)/chain.sample.rec(1,2) * 100 ,'%#6.2f') , ' %', ...
                     ' - MH acc = ' , num2str( chain.sample.rec(2,1)/chain.sample.rec(2,2) * 100 ,'%#6.2f') , ' %', ...
                     ' - FL acc = ' , num2str( chain.sample.rec(3,1)/chain.sample.rec(3,2) * 100 ,'%#6.2f') , ' %', ...
                     ])
            end
            
            r = r+1;
        end
    end    

    if flag_status
        disp(['CHAINER: chain expanded (total time = ',num2str(toc(tic_id)),' s)'])
    end


% reduce chain ------------------------------------------------------------
elseif d_length < 0
    
    d_length = -d_length;

    chain.params = chain_init.params;
    chain.length = d_length;
    chain.sizeGB = nan;
    chain.sample = chain_init.sample;
    
    ind = mod(chain_init.length,d_length)+(floor(chain_init.length/d_length)*(1:d_length));

    chain.i     = chain_init.i    (ind  );
    chain.sn    = chain_init.sn   (ind,:);
    chain.fn_D  = chain_init.fn_D (ind,:);
    chain.fn_A  = chain_init.fn_A (ind,:);
    chain.wi_D  = chain_init.wi_D (ind,:);
    chain.wi_A  = chain_init.wi_A (ind,:);
    chain.pm    = chain_init.pm   (ind,:);
    chain.ps    = chain_init.ps   (ind,:);
    chain.ksi_D = chain_init.ksi_D(ind  );
    chain.ksi_A = chain_init.ksi_A(ind  );
    chain.lam_S = chain_init.lam_S(ind  );
    chain.lam_D = chain_init.lam_D(ind,:);
    chain.lam_A = chain_init.lam_A(ind,:);
    chain.eff   = chain_init.eff  (ind,:);
 
    chain.params.i_skip = chain.i(2)-chain.i(1);
    
    if flag_status
        disp('CHAINER: chain reduced')
    end
end


%% save size
chain.sizeGB = get_sizeGB(chain);

end





%% auxiliary functions

function sizeGB = get_sizeGB(chain)
    sizeGB = whos( inputname(1) );
    sizeGB = sizeGB.bytes/1024^3;
end


function str = get_int_type(L)
    if     L <= intmax('uint8')
        str = 'uint8';
    elseif L <= intmax('uint16')
        str = 'uint16';
    elseif L <= intmax('uint32')
        str = 'uint32';
    elseif L <= intmax('uint64')
        str = 'uint64';
    else
        error('L way too large !!!');
    end
end
    



