function Gim = chainer_visualize(Gim,chain)


%% init
df = 0.02;

if isempty(Gim)
    
    col_M = [0 0 1];   % main molecule color
    col_D = [0 1 0];   % main donor color
    col_A = [1 0 0];   % main acceptor color
    col_m = [0 1 1];   % aux molecule color
    col_d = [0.8 1.0 0.8];   % aux donor color
    col_a = [1.0 0.5 0.5];   % aux acceptor color
    
    i = double(double(chain.i(1))) + double(chain.params.i_skip)*(0:chain.length-1)';
    t = (0.5:1:chain.params.N)*chain.params.dt;
    
    show_labels = false;

    num = 4;
    mum = 3;
    
    figure(10)
%    set(gcf,'windowstyle','docked')
    clf

    %----------------------------------------------------------------------
    ax_SAMP_1 = subplot(num,mum,[1 mum-1]);
    Gim.ax_SAMP_1{1} = plot(t,chain.sample.fn_D+df,'.-',...
                            t,chain.sample.fn_A-df,'.-');
    Gim.ax_SAMP_1{1}(1).Color = col_D;
    Gim.ax_SAMP_1{1}(2).Color = col_A;
    ylabel('Photo-state')
    ylim([-0.2 1.6])
    xlim([0 chain.params.N*chain.params.dt])
    set(gca,'YTick',[0 1],'YTickLabel',{'Dark','Bright'})
    legend('Donor (MCMC)','Acceptor (MCMC)','location','NorthEast','orientation','horizontal')
    legend boxoff
    box off
    title('Current sample')
    if show_labels 
        title('ax_SAMP_1','interpreter','none')
    end

    
    
    %----------------------------------------------------------------------
    ax_SAMP_2 = subplot(num,mum,[mum+1 (num-1)*mum-1]);
    Gim.ax_SAMP_2{1} = plot(t,chain.params.In_D/chain.params.dD,...
                            t,chain.params.In_A/chain.params.dD,...
                            t,chain.sample.lam_D(chain.sample.sn),'.-',...
                            t,chain.sample.lam_A(chain.sample.sn),'.-',...
                            [t(1) t(end)],chain.sample.ksi_D*[1 1],'--',...
                            [t(1) t(end)],chain.sample.ksi_A*[1 1],'--',...
                            [t(1) t(end)],chain.sample.lam_S*[1 1],':');
    ylim([-0.25 1.25]*max([chain.params.In_D,chain.params.In_A]/chain.params.dD))
    xlim([0 chain.params.N*chain.params.dt])
    Gim.ax_SAMP_2{1}(1).Color = col_d;
    Gim.ax_SAMP_2{1}(2).Color = col_a;
    Gim.ax_SAMP_2{1}(3).Color = col_D;
    Gim.ax_SAMP_2{1}(4).Color = col_A;
    Gim.ax_SAMP_2{1}(5).Color = col_D;
    Gim.ax_SAMP_2{1}(6).Color = col_A;
    Gim.ax_SAMP_2{1}(7).Color = col_d;
    ylabel(['Emission rate (',chain.params.units_I,'/',chain.params.units_t,')'])
    legend(Gim.ax_SAMP_2{1}(3:4),'Donor (MCMC)','Acceptor (MCMC)','location','NorthEast','orientation','vertical')
    legend boxoff
    box off
    grid on
    if show_labels 
        title('ax_SAMP_2','interpreter','none')
    end

    axes('Position',ax_SAMP_2.Position);
    set(gca, 'Color','none', 'XTick',[], 'YTick',[],'Box','off')
    legend(gca,Gim.ax_SAMP_2{1}(1:2),{'Donor (apparent)','Acceptor (apparent)'},'location','NorthWest','orientation','vertical','color','w')
    legend boxoff

    axes('Position',ax_SAMP_2.Position);
    set(gca, 'Color','none', 'XTick',[], 'YTick',[],'Box','off')
    legend(gca,Gim.ax_SAMP_2{1}(5:7),{'Donor background','Acceptor background','Donor no-FRET'},'location','SouthEast','orientation','vertical','color','w')
    legend boxoff
    
    
    %----------------------------------------------------------------------
    ax_SAMP_3 = subplot(num,mum,[(num-1)*mum+1 num*mum-1]);
    Gim.ax_SAMP_3{1} = plot(t,100*chain.params.In_A./(chain.params.In_D+chain.params.In_A+eps),...
                            t,100*chain.sample.eff(chain.sample.sn),'.-');
    ylim([-0.2 1.5]*100)
    xlim([0 chain.params.N*chain.params.dt])
    Gim.ax_SAMP_3{1}(1).Color = col_m;
    Gim.ax_SAMP_3{1}(2).Color = col_M;
    ylabel('FRET efficiency (%)')
    xlabel(['Time (',chain.params.units_t,')'])
    legend('Apparent','MCMC','location','NorthEast','orientation','horizontal')
    legend boxoff
    box off
    grid on
    if show_labels 
        title('ax_SAMP_3','interpreter','none')
    end

    
    
    
    %----------------------------------------------------------------------
    %----------------------------------------------------------------------
    ax_MCMC_1 = subplot(num,mum,[2*mum (num-1)*mum]);
    Gim.ax_MCMC_1{1} = plot( reshape(i*ones(1,chain.params.M),[],1), reshape(chain.lam_D,[],1),'.',...
                             reshape(i*ones(1,chain.params.M),[],1), reshape(chain.lam_A,[],1),'.',...
                             i,chain.ksi_D,'.-',...
                             i,chain.ksi_A,'.-',...
                             i,chain.lam_S,'.-');
    Gim.ax_MCMC_1{2} = line(chain.sample.i*[1 1],get(gca,'Ylim'),'color','k','linestyle','--');
    Gim.ax_MCMC_1{1}(1).Color = col_D;
    Gim.ax_MCMC_1{1}(2).Color = col_A;
    Gim.ax_MCMC_1{1}(3).Color = col_d;
    Gim.ax_MCMC_1{1}(4).Color = col_a;
    Gim.ax_MCMC_1{1}(5).Color = col_d;
    ylim( ax_SAMP_2.YLim )
    xlim([max(0,i(1)-1),i(end)+1])
    h = show_prior(gca,'besselK',[chain.params.tht_prior_phi,chain.params.kap_D_prior_phi],[chain.params.tht_prior_psi,chain.params.kap_D_prior_psi]); h.Color = col_D;
    h = show_prior(gca,'besselK',[chain.params.tht_prior_phi,chain.params.kap_A_prior_phi],[chain.params.tht_prior_psi,chain.params.kap_A_prior_psi]); h.Color = col_A;
    h = show_prior(gca,'besselK',[chain.params.tht_prior_phi,chain.params.rho_D_prior_phi],[chain.params.tht_prior_psi,chain.params.rho_D_prior_psi]); h.Color = col_d; h.LineStyle = ':';
    h = show_prior(gca,'besselK',[chain.params.tht_prior_phi,chain.params.rho_A_prior_phi],[chain.params.tht_prior_psi,chain.params.rho_A_prior_psi]); h.Color = col_a; h.LineStyle = ':';
    h = show_prior(gca,'besselK',[chain.params.tht_prior_phi,chain.params.kap_S_prior_phi],[chain.params.tht_prior_psi,chain.params.kap_S_prior_psi]); h.Color = col_d; h.LineStyle = ':';
    set(gca,'YAxisLocation','Right')
    ylabel(['Emission rates (',chain.params.units_I,'/',chain.params.units_t,')'])
    title('MCMC samples')
    box off
    grid on
    if show_labels 
        title('ax_MCMC_1','interpreter','none')
    end

    
    
    %----------------------------------------------------------------------
    ax_MCMC_2 = subplot(num,mum,num*mum);
    Gim.ax_MCMC_2{1} = plot( reshape(i*ones(1,chain.params.M),[],1), reshape(100*chain.eff,[],1),'.');
    Gim.ax_MCMC_2{2} = line(chain.sample.i*[1 1],get(gca,'Ylim'),'color','k','linestyle','--');
    Gim.ax_MCMC_2{1}.Color = col_M;
    ylim( ax_SAMP_3.YLim )
    xlim([max(0,i(1)-1),i(end)+1])
    h = show_prior(gca,'eff_custom',[chain.params.kap_D_prior_phi,chain.params.kap_D_prior_psi],...
                                    [chain.params.kap_A_prior_phi,chain.params.kap_A_prior_psi]); h.Color = col_M;
    set(gca,'YAxisLocation','Right')
    ylabel('FRET efficiency (%)')
    xlabel('MCMC iteration (i)')
    box off
    grid on
    if show_labels 
        title('ax_MCMC_2','interpreter','none')
    end

    
end


%%
Gim.ax_SAMP_1{1}(1).YData = double( chain.sample.fn_D+df );
Gim.ax_SAMP_1{1}(2).YData = double( chain.sample.fn_A-df );
    
Gim.ax_SAMP_2{1}(3).YData = chain.sample.lam_D(chain.sample.sn);
Gim.ax_SAMP_2{1}(4).YData = chain.sample.lam_A(chain.sample.sn);
Gim.ax_SAMP_2{1}(5).YData = chain.sample.ksi_D*[1 1];
Gim.ax_SAMP_2{1}(6).YData = chain.sample.ksi_A*[1 1];
Gim.ax_SAMP_2{1}(7).YData = chain.sample.lam_S*[1 1];

Gim.ax_SAMP_3{1}(2).YData = 100*chain.sample.eff(chain.sample.sn);

Gim.ax_MCMC_1{1}(1).YData = reshape(chain.lam_D,1,[]);
Gim.ax_MCMC_1{1}(2).YData = reshape(chain.lam_A,[],1);
Gim.ax_MCMC_1{1}(3).YData = chain.ksi_D;
Gim.ax_MCMC_1{1}(4).YData = chain.ksi_A;
Gim.ax_MCMC_1{1}(5).YData = chain.lam_S;
Gim.ax_MCMC_1{2}.XData = chain.sample.i*[1 1];

Gim.ax_MCMC_2{1}.YData = reshape(100*chain.eff,[],1);
Gim.ax_MCMC_2{2}.XData = chain.sample.i*[1 1];


drawnow





%%
function h = show_prior(ax,tag,p1,p2)

% RV
x_temp = linspace(min(get(ax,'Ylim')),max(get(ax,'Ylim')));

% PDF
switch tag
    case 'norm' % p1 = mu, p2 = sg
        p_temp = normpdf(x_temp,p1,p2)+eps; % PDF
    case 'symnorm' % p1 = mu, p2 = sg
        p_temp = 0.5*normpdf(x_temp,p1,p2)+0.5*normpdf(x_temp,-p1,p2)+eps; % PDF
    case 'gamma' % p1 = a, p2 = b
        p_temp = gampdf(x_temp,p1,p2); % PDF
    case 'eff_custom' % p1 = [phi_D,psi_D], p2 = [phi_A,psi_A]
        x_temp = x_temp/100;
        p_temp = (p2(2)/p1(2)*p1(1)/p2(1))^p1(1)/beta(p1(1),p2(1))*(1./x_temp-1).^(p1(1)-1).*(1+p2(2)/p1(2)*p2(1)/p1(1)*(1./x_temp-1)).^(-p1(1)-p2(1))./x_temp.^2;
        p_temp(x_temp<0) = 0;
        p_temp(x_temp>1) = 0;
        x_temp = x_temp*100;
    case 'besselK' % p1(1:2) = phi(1:2), p2 = psi(1:2)
        p_temp = eps + 2/(gamma(p1(1))*gamma(p1(2)))*(p1(1)*p1(2)/(p2(1)*p2(2))*x_temp).^(0.5*(p1(1)+p1(2))).*besselk(p1(1)-p1(2),2*sqrt(p1(1)*p1(2)/(p2(1)*p2(2))*x_temp))./x_temp;
        p_temp(x_temp<0) = 0;
    otherwise
        warning('Unknown tag (prior visualizer)')
end

% print
p_temp = min(get(ax,'Xlim')) + 0.15*(max(get(ax,'Xlim'))-min(get(ax,'Xlim')))*p_temp/max(p_temp);
h = line(p_temp,x_temp,'color','k','linestyle','--','parent',ax);


