function show_estimates_photo_physics(chain,fr,dr,win)

idx = ceil(fr*chain.length):dr:chain.length;

w0_D_mcmc = 1-chain.wi_D(idx,1);    % 0 -> 0
w1_D_mcmc =   chain.wi_D(idx,2);    % 1 -> 1
w0_A_mcmc = 1-chain.wi_A(idx,1);    % 0 -> 0
w1_A_mcmc =   chain.wi_A(idx,2);    % 1 -> 1
               

%%
figure(win)
clf

w_bnd = linspace(0,1,175);

subplot(2,3,1)
h1 = histogram(w0_D_mcmc,w_bnd,'normalization','pdf','facecolor','y');
hold on
h2 = histogram(w1_D_mcmc,w_bnd,'normalization','pdf','facecolor','g');
hold off
xlim([0 1])
box off
ylabel('Probability density')
xlabel('Self-transition probability \omega')
legend([h1,h2],'\omega^D_0','\omega^D_1','orientation','horizontal','location','north')
legend boxoff

subplot(2,3,4)
h1 = histogram(w0_A_mcmc,w_bnd,'normalization','pdf','facecolor','m');
hold on
h2 = histogram(w1_A_mcmc,w_bnd,'normalization','pdf','facecolor','r');
hold off
xlim([0 1])
box off
ylabel('Probability density')
xlabel('Self-transition probability \omega')
legend([h1,h2],'\omega^A_0','\omega^A_1','orientation','horizontal','location','north')
legend boxoff





t_bnd = logspace(log10(0.5*chain.params.dt),log10(2*chain.params.N*chain.params.dt),75);


subplot(2,3,[2 3])
h1 = histogram(chain.params.dt./(1-w0_D_mcmc),t_bnd,'normalization','probability','facecolor','y');
hold on
h2 = histogram(chain.params.dt./(1-w1_D_mcmc),t_bnd,'normalization','probability','facecolor','g');
hold off
xlim([0.5 2*chain.params.N]*chain.params.dt)
box off
ylabel('Probability (norm.)')
xlabel(['Mean dwell time \tau (',chain.params.units_t,')'])
set(gca,'XScale','Log')
line(chain.params.dt*[1 1],get(gca,'Ylim'),'color','k','linestyle','--')
line(chain.params.N*chain.params.dt*[1 1],get(gca,'Ylim'),'color','k','linestyle','--')
legend([h1,h2],'\tau^D_0','\tau^D_1','orientation','horizontal','location','north')
legend boxoff


subplot(2,3,[5 6])
h1 = histogram(chain.params.dt./(1-w0_A_mcmc),t_bnd,'normalization','probability','facecolor','m');
hold on
h2 = histogram(chain.params.dt./(1-w1_A_mcmc),t_bnd,'normalization','probability','facecolor','r');
hold off
xlim([0.5 2*chain.params.N]*chain.params.dt)
box off
ylabel('Probability (norm.)')
xlabel(['Mean dwell time \tau (',chain.params.units_t,')'])
set(gca,'XScale','Log')
line(chain.params.dt*[1 1],get(gca,'Ylim'),'color','k','linestyle','--')
line(chain.params.N*chain.params.dt*[1 1],get(gca,'Ylim'),'color','k','linestyle','--')
legend([h1,h2],'\tau^A_0','\tau^A_1','orientation','horizontal','location','north')
legend boxoff

