function [tht,rho_D,rho_A,kap_D,kap_A,kap_S, rec ] = update_emissions( fn_D, fn_A, sn, ...
                                                                      rho_D,rho_A, ...
                                                                      kap_D,kap_A,kap_S, rec, params)

tht_phi_post = params.tht_prior_phi + sum(params.In_D+params.In_A);

[S,~,iS] = unique(sn);
B = length(S);

for rep = 1:params.rep_tht
    % update tht (direct)
    tht = params.tht_prior_phi/params.tht_prior_psi ...
        + params.dD*( params.N*(params.qD*rho_D+params.qA*rho_A) ...
                    + (params.qD*params.cDD+params.qA*params.cDA)*sum( fn_D.*(fn_A.*kap_D(sn)+(1-fn_A)*kap_S) ) ...
                    + (params.qD*params.cAD+params.qA*params.cAA)*sum( fn_D.* fn_A.*kap_A(sn) ) );
    tht = randg(tht_phi_post)/tht;

    % update background and active multipliers (HMC)
    [rho_D,rho_A,kap_D(S),kap_A(S),kap_S,rec(1,:)] = HMC(rho_D,rho_A,kap_D(S),kap_A(S),kap_S,tht,fn_D,fn_A,iS,B,rec(1,:),params);
    
    % update background and active multipliers (MH)
    [rho_D,rho_A,kap_D(S),kap_A(S),kap_S,rec(2,:)] = MH(rho_D,rho_A,kap_D(S),kap_A(S),kap_S,tht,fn_D,fn_A,iS,rec(2,:),params);

    % flip donor active multipliers (MH)
    [kap_D,kap_S,rec(3,:)] = fliper(rho_D,rho_A,kap_D,kap_A,kap_S,tht,fn_D,fn_A,iS,rec(3,:),params);
end

% update inactive multipliers (direct)
S = setdiff(1:params.M,S);
B = length(S);
kap_D(S) = gamrnd( params.kap_D_prior_phi, params.kap_D_prior_psi/params.kap_D_prior_phi , 1, B );
kap_A(S) = gamrnd( params.kap_A_prior_phi, params.kap_A_prior_psi/params.kap_A_prior_phi , 1, B );

end







%% HMC updates
function [sample_rD,sample_rA,sample_kD,sample_kA,sample_kS,rec] = HMC(sample_rD,sample_rA,sample_kD,sample_kA,sample_kS,t,fD,fA,iS,B,rec,params)

HMC_eps  = params.HMC_eps*rand(1);
HMC_L    = params.HMC_L;
HMC_demo = false;

% masses
m_RD = 1;
m_RA = 1;
m_KD = ones(1,B);
m_KA = ones(1,B);
m_KS = 1;


% momenta
sample_RD = m_RD *randn(1);
sample_RA = m_RA *randn(1);
sample_KD = m_KD.*randn(1,B);
sample_KA = m_KA.*randn(1,B);
sample_KS = m_KS.*randn(1);


% pre-allocation
propos_rD = nan(HMC_L,1);
propos_rA = nan(HMC_L,1);
propos_kD = nan(HMC_L,B);
propos_kA = nan(HMC_L,B);
propos_kS = nan(HMC_L,1);

propos_RD = nan(HMC_L,1);
propos_RA = nan(HMC_L,1);
propos_KD = nan(HMC_L,B);
propos_KA = nan(HMC_L,B);
propos_KS = nan(HMC_L,1);


% init step
i = 1;
propos_rD(i  ) = sample_rD;
propos_rA(i  ) = sample_rA;
propos_kD(i,:) = sample_kD;
propos_kA(i,:) = sample_kA;
propos_kS(i  ) = sample_kS;

[U_rD,U_rA,U_kD,U_kA,U_kS] = get_U_grad(sample_rD,sample_rA,sample_kD,sample_kA,sample_kS,t,fD,fA,iS,B,params);
propos_RD(i  ) = sample_RD - 0.5*HMC_eps*U_rD;
propos_RA(i  ) = sample_RA - 0.5*HMC_eps*U_rA;
propos_KD(i,:) = sample_KD - 0.5*HMC_eps*U_kD;
propos_KA(i,:) = sample_KA - 0.5*HMC_eps*U_kA;
propos_KS(i  ) = sample_KS - 0.5*HMC_eps*U_kS;

if HMC_demo; Gim = HMC_plot([],B,HMC_L, propos_rD,propos_rA,propos_kD,propos_kA,propos_kS, propos_RD,propos_RA,propos_KD,propos_KA,propos_KS); end


% leap-frog forward
for i = 2:HMC_L-1
    propos_rD(i  ) = propos_rD(i-1  ) + HMC_eps*propos_RD(i-1  ) /m_RD;
    propos_rA(i  ) = propos_rA(i-1  ) + HMC_eps*propos_RA(i-1  ) /m_RA;
    propos_kD(i,:) = propos_kD(i-1,:) + HMC_eps*propos_KD(i-1,:)./m_KD;
    propos_kA(i,:) = propos_kA(i-1,:) + HMC_eps*propos_KA(i-1,:)./m_KA;
    propos_kS(i  ) = propos_kS(i-1  ) + HMC_eps*propos_KS(i-1  ) /m_KS;
    
    [U_rD,U_rA,U_kD,U_kA,U_kS] = get_U_grad(propos_rD(i),propos_rA(i),propos_kD(i,:),propos_kA(i,:),propos_kS(i),t,fD,fA,iS,B,params);
    propos_RD(i  ) = propos_RD(i-1  ) - HMC_eps*U_rD;
    propos_RA(i  ) = propos_RA(i-1  ) - HMC_eps*U_rA;
    propos_KD(i,:) = propos_KD(i-1,:) - HMC_eps*U_kD;
    propos_KA(i,:) = propos_KA(i-1,:) - HMC_eps*U_kA;
    propos_KS(i  ) = propos_KS(i-1  ) - HMC_eps*U_kS;

    if HMC_demo; HMC_plot(Gim,B,[], propos_rD,propos_rA,propos_kD,propos_kA,propos_kS, propos_RD,propos_RA,propos_KD,propos_KA,propos_KS); end
end

% term step
i = HMC_L;
propos_rD(i  ) = propos_rD(i-1  ) + HMC_eps*propos_RD(i-1  ) /m_RD;
propos_rA(i  ) = propos_rA(i-1  ) + HMC_eps*propos_RA(i-1  ) /m_RA;
propos_kD(i,:) = propos_kD(i-1,:) + HMC_eps*propos_KD(i-1,:)./m_KD;
propos_kA(i,:) = propos_kA(i-1,:) + HMC_eps*propos_KA(i-1,:)./m_KA;
propos_kS(i  ) = propos_kS(i-1  ) + HMC_eps*propos_KS(i-1  ) /m_KS;

[U_rD,U_rA,U_kD,U_kA,U_kS] = get_U_grad(propos_rD(i),propos_rA(i),propos_kD(i,:),propos_kA(i,:),propos_kS(i),t,fD,fA,iS,B,params);
propos_RD(i  ) = propos_RD(i-1  ) - 0.5*HMC_eps*U_rD;
propos_RA(i  ) = propos_RA(i-1  ) - 0.5*HMC_eps*U_rA;
propos_KD(i,:) = propos_KD(i-1,:) - 0.5*HMC_eps*U_kD;
propos_KA(i,:) = propos_KA(i-1,:) - 0.5*HMC_eps*U_kA;
propos_KS(i  ) = propos_KS(i-1  ) - 0.5*HMC_eps*U_kS;

if HMC_demo; HMC_plot(Gim,B,[], propos_rD,propos_rA,propos_kD,propos_kA,propos_kS, propos_RD,propos_RA,propos_KD,propos_KA,propos_KS); end


log_a = get_U(sample_rD,     sample_rA     ,sample_kD       ,sample_kA       ,sample_kS     ,t,fD,fA,iS,params) ...
      - get_U(propos_rD(end),propos_rA(end),propos_kD(end,:),propos_kA(end,:),propos_kS(end),t,fD,fA,iS,params) ...
      + 0.5*( (sample_RD ^2-propos_RD(end  ) ^2) /m_RD ...
            + (sample_RA ^2-propos_RA(end  ) ^2) /m_RA ...
            + (sample_KD.^2-propos_KD(end,:).^2)./m_KD ...
            + (sample_KA.^2-propos_KA(end,:).^2)./m_KA ...
            + (sample_KS ^2-propos_KS(end  ) ^2) /m_KS );

if log(rand) < log_a
    sample_rD = propos_rD(end  );
    sample_rA = propos_rA(end  );
    sample_kD = propos_kD(end,:);
    sample_kA = propos_kA(end,:);
    sample_kS = propos_kS(end  );
    rec(1) = rec(1)+1;
end
    rec(2) = rec(2)+1;

    

end


%% ploter
function Gim = HMC_plot(Gim,B,HMC_L,propos_rD,propos_rA,propos_kD,propos_kA,propos_kS, ...
                                    propos_RD,propos_RA,propos_KD,propos_KA,propos_KS)

if isempty(Gim)
    figure(20)
    
    col = lines(2);
    
    subplot(2,3,[1 2])
    Gim.ax_Di{1} = plot(1:HMC_L,propos_kD,'o-'        ,'color',col(1,:));
    xlim([0 HMC_L+1])
    Gim.ax_Di{2} = line(1:HMC_L,propos_rD,'marker','d','color',col(2,:));
    
    subplot(2,3,3)
    Gim.ax_Dm{1} = plot(propos_KD,propos_kD,'o-'        ,'color',col(1,:));
    Gim.ax_Dm{2} = line(propos_RD,propos_rD,'marker','d','color',col(2,:));


    subplot(2,3,[4 5])
    Gim.ax_Ai{1} = plot(1:HMC_L,propos_kA,'o-'        ,'color',col(1,:));
    xlim([0 HMC_L+1])
    Gim.ax_Ai{2} = line(1:HMC_L,propos_rA,'marker','d','color',col(2,:));
    
    subplot(2,3,6)
    Gim.ax_Am{1} = plot(propos_KA,propos_kA,'o-'        ,'color',col(1,:));
    Gim.ax_Am{2} = line(propos_RA,propos_rA,'marker','d','color',col(2,:));
end

for m=1:B
    Gim.ax_Di{1}(m).YData = propos_kD(:,m);
    Gim.ax_Dm{1}(m).XData = propos_KD(:,m);
    Gim.ax_Dm{1}(m).YData = propos_kD(:,m);

    Gim.ax_Ai{1}(m).YData = propos_kA(:,m);
    Gim.ax_Am{1}(m).XData = propos_KA(:,m);
    Gim.ax_Am{1}(m).YData = propos_kA(:,m);
end

Gim.ax_Di{2}.YData = propos_rD;
Gim.ax_Dm{2}.XData = propos_RD;
Gim.ax_Dm{2}.YData = propos_rD;

Gim.ax_Ai{2}.YData = propos_rA;
Gim.ax_Am{2}.XData = propos_RA;
Gim.ax_Am{2}.YData = propos_rA;

drawnow

end


%% HMC potential
function U = get_U(rD,rA,kD,kA,kS,t,fD,fA,S,params)


if (rD<0)||(rA<0)||any(kD<0)||any(kA<0)||any(kS<0)
    U = inf;
else
    hat_D = rD + params.cDD*fD.*( fA.*kD(S')+(1-fA)*kS ) + params.cAD*fD.*fA.*kA(S');
    hat_A = rA + params.cDA*fD.*( fA.*kD(S')+(1-fA)*kS ) + params.cAA*fD.*fA.*kA(S');

    U = t*params.dD*( params.qD*sum(hat_D) + params.qA*sum(hat_A) ) ...
      - sum( params.In_D.*log(hat_D) + params.In_A.*log(hat_A)  ) ...    
      + (1-params.rho_D_prior_phi)*     log(rD)   + params.rho_D_prior_phi/params.rho_D_prior_psi*     rD   ...
      + (1-params.rho_A_prior_phi)*     log(rA)   + params.rho_A_prior_phi/params.rho_A_prior_psi*     rA   ...
      + (1-params.kap_D_prior_phi)*sum( log(kD) ) + params.kap_D_prior_phi/params.kap_D_prior_psi*sum( kD ) ...
      + (1-params.kap_A_prior_phi)*sum( log(kA) ) + params.kap_A_prior_phi/params.kap_A_prior_psi*sum( kA ) ...
      + (1-params.kap_S_prior_phi)*     log(kS)   + params.kap_S_prior_phi/params.kap_S_prior_psi*     kS;
end

end
    
%% HMC potential gradient
function [U_rD,U_rA,U_kD,U_kA,U_kS] = get_U_grad(rD,rA,kD,kA,kS,t,fD,fA,S,B,params)


ID_hat_D = params.In_D./( rD + params.cDD*fD.*( fA.*kD(S')+(1-fA)*kS ) + params.cAD*fD.*fA.*kA(S') );
IA_hat_A = params.In_A./( rA + params.cDA*fD.*( fA.*kD(S')+(1-fA)*kS ) + params.cAA*fD.*fA.*kA(S') );

U_rD = - sum(ID_hat_D) ...
       + params.qD*t*params.dD*params.N ...
       + (1-params.rho_D_prior_phi)/rD ...
       + params.rho_D_prior_phi/params.rho_D_prior_psi;

U_rA = - sum(IA_hat_A) ...
       + params.qA*t*params.dD*params.N ...
       + (1-params.rho_A_prior_phi)/rA ...
       + params.rho_A_prior_phi/params.rho_A_prior_psi;


U_kD = nan(1,B);
U_kA = nan(1,B);

for m = 1:B
    idx = find( (S==m)&(fD==1)'&(fA==1)' );

    U_kD(m) = - params.cDD*sum(ID_hat_D(idx)) ...
              - params.cDA*sum(IA_hat_A(idx)) ...
              + (params.qD*params.cDD+params.qA*params.cDA)*t*params.dD*length(idx) ...
              + (1-params.kap_D_prior_phi)/kD(m) ...
              + params.kap_D_prior_phi/params.kap_D_prior_psi;

    U_kA(m) = - params.cAD*sum(ID_hat_D(idx)) ...
              - params.cAA*sum(IA_hat_A(idx)) ...
              + (params.qD*params.cAD+params.qA*params.cAA)*t*params.dD*length(idx) ...
              + (1-params.kap_A_prior_phi)/kA(m) ...
              + params.kap_A_prior_phi/params.kap_A_prior_psi;
end

idx = find( (fD==1)'&(fA==0)' );

U_kS = - params.cDD*sum(ID_hat_D(idx)) ...
       - params.cDA*sum(IA_hat_A(idx)) ...
       + (params.qD*params.cDD+params.qA*params.cDA)*t*params.dD*length(idx) ...
       + (1-params.kap_S_prior_phi)/kS ...
       + params.kap_S_prior_phi/params.kap_S_prior_psi;

end




%% MH updates (translations)
function [sample_rho_D,sample_rho_A,sample_kap_D,sample_kap_A,sample_kap_S,rec] = MH(sample_rho_D,sample_rho_A,sample_kap_D,sample_kap_A,sample_kap_S,tht,fn_D,fn_A,S,rec,params)

a_D = params.MH_a_D;
a_A = params.MH_a_A;


sample_mu_D = params.qD*tht*params.dD*( sample_rho_D + params.cDD*fn_D.*( fn_A.*sample_kap_D(S')+(1-fn_A)*sample_kap_S ) + params.cAD*fn_D.*fn_A.*sample_kap_A(S') );
sample_mu_A = params.qA*tht*params.dD*( sample_rho_A + params.cDA*fn_D.*( fn_A.*sample_kap_D(S')+(1-fn_A)*sample_kap_S ) + params.cAA*fn_D.*fn_A.*sample_kap_A(S') );

for rep = 1:params.rep_rho
    
    switch randi(3)
        case 1
            propos_rho_D = sample_rho_D/a_D * randg(a_D);
            propos_rho_A = sample_rho_A/a_A * randg(a_A);
        case 2
            propos_rho_D = sample_rho_D;
            propos_rho_A = sample_rho_A/a_A * randg(a_A);
        case 3
            propos_rho_D = sample_rho_D/a_D * randg(a_D);
            propos_rho_A = sample_rho_A;
    end
    d = [params.cDD,params.cAD;params.cDA,params.cAA]\[sample_rho_D-propos_rho_D; sample_rho_A-propos_rho_A];
    propos_kap_D = sample_kap_D + d(1);
    propos_kap_A = sample_kap_A + d(2);
    propos_kap_S = sample_kap_S + d(1);

    propos_mu_D = params.qD*tht*params.dD*( propos_rho_D + params.cDD*fn_D.*( fn_A.*propos_kap_D(S')+(1-fn_A)*propos_kap_S ) + params.cAD*fn_D.*fn_A.*propos_kap_A(S') );
    propos_mu_A = params.qA*tht*params.dD*( propos_rho_A + params.cDA*fn_D.*( fn_A.*propos_kap_D(S')+(1-fn_A)*propos_kap_S ) + params.cAA*fn_D.*fn_A.*propos_kap_A(S') );
    
    if all(propos_kap_D>0) && all(propos_kap_A>0) && (propos_kap_S>0)
        log_a = sum( (sample_mu_D-propos_mu_D) + (sample_mu_A-propos_mu_A) ...
                   + params.In_D.*log(propos_mu_D./sample_mu_D) ...
                   + params.In_A.*log(propos_mu_A./sample_mu_A) ) ...
                   + (params.rho_D_prior_phi-1)*    log(propos_rho_D/sample_rho_D)  + params.rho_D_prior_phi*   (sample_rho_D-propos_rho_D)/params.rho_D_prior_psi ...
                   + (params.rho_A_prior_phi-1)*    log(propos_rho_A/sample_rho_A)  + params.rho_A_prior_phi*   (sample_rho_A-propos_rho_A)/params.rho_A_prior_psi ...
                   + (params.kap_D_prior_phi-1)*sum(log(propos_kap_D/sample_kap_D)) + params.kap_D_prior_phi*sum(sample_kap_D-propos_kap_D)/params.kap_D_prior_psi ...
                   + (params.kap_A_prior_phi-1)*sum(log(propos_kap_A/sample_kap_A)) + params.kap_A_prior_phi*sum(sample_kap_A-propos_kap_A)/params.kap_A_prior_psi ...
                   + (params.kap_S_prior_phi-1)*    log(propos_kap_S/sample_kap_S)  + params.kap_S_prior_phi*   (sample_kap_S-propos_kap_S)/params.kap_S_prior_psi ...
                   + (2*a_D-1)*log(sample_rho_D/propos_rho_D) + a_D*( propos_rho_D/sample_rho_D - sample_rho_D/propos_rho_D ) ...
                   + (2*a_A-1)*log(sample_rho_A/propos_rho_A) + a_A*( propos_rho_A/sample_rho_A - sample_rho_A/propos_rho_A );
    else
        log_a = -inf;
    end
    
    
    if log(rand) < log_a
        sample_rho_D = propos_rho_D;
        sample_rho_A = propos_rho_A;
        sample_kap_D = propos_kap_D;
        sample_kap_A = propos_kap_A;
        sample_kap_S = propos_kap_S;
        sample_mu_D = propos_mu_D;
        sample_mu_A = propos_mu_A;
        rec(1) = rec(1)+1;
    end
        rec(2) = rec(2)+1;
    
end

end






%% MH updates (flips)
function [sample_kap_D,sample_kap_S,rec] = fliper(rho_D,rho_A,sample_kap_D,kap_A,sample_kap_S,tht,fn_D,fn_A,S,rec,params)

sample_mu_D = params.qD*tht*params.dD*( rho_D + params.cDD*fn_D.*( fn_A.*sample_kap_D(S')+(1-fn_A)*sample_kap_S ) + params.cAD*fn_D.*fn_A.*kap_A(S') );
sample_mu_A = params.qA*tht*params.dD*( rho_A + params.cDA*fn_D.*( fn_A.*sample_kap_D(S')+(1-fn_A)*sample_kap_S ) + params.cAA*fn_D.*fn_A.*kap_A(S') );


m = randi(params.M);
propos_kap_D    = sample_kap_D;
propos_kap_D(m) = sample_kap_S;
propos_kap_S    = sample_kap_D(m);

propos_mu_D = params.qD*tht*params.dD*( rho_D + params.cDD*fn_D.*( fn_A.*propos_kap_D(S')+(1-fn_A)*propos_kap_S ) + params.cAD*fn_D.*fn_A.*kap_A(S') );
propos_mu_A = params.qA*tht*params.dD*( rho_A + params.cDA*fn_D.*( fn_A.*propos_kap_D(S')+(1-fn_A)*propos_kap_S ) + params.cAA*fn_D.*fn_A.*kap_A(S') );

log_a = sum( (sample_mu_D-propos_mu_D) + (sample_mu_A-propos_mu_A) ...
           + params.In_D.*log(propos_mu_D./sample_mu_D) ...
           + params.In_A.*log(propos_mu_A./sample_mu_A) ) ...
           + (params.kap_D_prior_phi-1)*log( propos_kap_D(m)/sample_kap_D(m) ) + params.kap_D_prior_phi*( sample_kap_D(m)-propos_kap_D(m) )/params.kap_D_prior_psi ...
           + (params.kap_S_prior_phi-1)*log( propos_kap_S   /sample_kap_S    ) + params.kap_S_prior_phi*( sample_kap_S   -propos_kap_S    )/params.kap_S_prior_psi;

if log(rand) < log_a
    sample_kap_D = propos_kap_D;
    sample_kap_S = propos_kap_S;
    rec(1) = rec(1)+1;
end
    rec(2) = rec(2)+1;

end




